# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.client_facing_diagnosis_information import ClientFacingDiagnosisInformation
from ..types.client_facing_payor_search_response import ClientFacingPayorSearchResponse
from ..types.client_facing_payor_search_response_deprecated import ClientFacingPayorSearchResponseDeprecated
from ..types.payor_code_external_provider import PayorCodeExternalProvider
from .raw_client import AsyncRawInsuranceClient, RawInsuranceClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InsuranceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInsuranceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInsuranceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInsuranceClient
        """
        return self._raw_client

    def search_get_payor_info(
        self,
        *,
        insurance_name: typing.Optional[str] = None,
        provider: typing.Optional[PayorCodeExternalProvider] = None,
        provider_payor_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ClientFacingPayorSearchResponse]:
        """
        Parameters
        ----------
        insurance_name : typing.Optional[str]

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_payor_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingPayorSearchResponse]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.insurance.search_get_payor_info()
        """
        _response = self._raw_client.search_get_payor_info(
            insurance_name=insurance_name,
            provider=provider,
            provider_payor_id=provider_payor_id,
            request_options=request_options,
        )
        return _response.data

    def search_payor_info(
        self,
        *,
        insurance_name: typing.Optional[str] = OMIT,
        provider: typing.Optional[PayorCodeExternalProvider] = OMIT,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ClientFacingPayorSearchResponseDeprecated]:
        """
        Parameters
        ----------
        insurance_name : typing.Optional[str]

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingPayorSearchResponseDeprecated]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.insurance.search_payor_info()
        """
        _response = self._raw_client.search_payor_info(
            insurance_name=insurance_name, provider=provider, provider_id=provider_id, request_options=request_options
        )
        return _response.data

    def search_diagnosis(
        self, *, diagnosis_query: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientFacingDiagnosisInformation]:
        """
        Parameters
        ----------
        diagnosis_query : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingDiagnosisInformation]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.insurance.search_diagnosis(diagnosis_query='diagnosis_query', )
        """
        _response = self._raw_client.search_diagnosis(diagnosis_query=diagnosis_query, request_options=request_options)
        return _response.data


class AsyncInsuranceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInsuranceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInsuranceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInsuranceClient
        """
        return self._raw_client

    async def search_get_payor_info(
        self,
        *,
        insurance_name: typing.Optional[str] = None,
        provider: typing.Optional[PayorCodeExternalProvider] = None,
        provider_payor_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ClientFacingPayorSearchResponse]:
        """
        Parameters
        ----------
        insurance_name : typing.Optional[str]

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_payor_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingPayorSearchResponse]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.insurance.search_get_payor_info()
        asyncio.run(main())
        """
        _response = await self._raw_client.search_get_payor_info(
            insurance_name=insurance_name,
            provider=provider,
            provider_payor_id=provider_payor_id,
            request_options=request_options,
        )
        return _response.data

    async def search_payor_info(
        self,
        *,
        insurance_name: typing.Optional[str] = OMIT,
        provider: typing.Optional[PayorCodeExternalProvider] = OMIT,
        provider_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[ClientFacingPayorSearchResponseDeprecated]:
        """
        Parameters
        ----------
        insurance_name : typing.Optional[str]

        provider : typing.Optional[PayorCodeExternalProvider]

        provider_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingPayorSearchResponseDeprecated]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.insurance.search_payor_info()
        asyncio.run(main())
        """
        _response = await self._raw_client.search_payor_info(
            insurance_name=insurance_name, provider=provider, provider_id=provider_id, request_options=request_options
        )
        return _response.data

    async def search_diagnosis(
        self, *, diagnosis_query: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientFacingDiagnosisInformation]:
        """
        Parameters
        ----------
        diagnosis_query : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingDiagnosisInformation]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.insurance.search_diagnosis(diagnosis_query='diagnosis_query', )
        asyncio.run(main())
        """
        _response = await self._raw_client.search_diagnosis(
            diagnosis_query=diagnosis_query, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PriorityResource(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    WORKOUTS = "workouts"
    ACTIVITY = "activity"
    SLEEP = "sleep"
    BODY = "body"
    TIMESERIES = "timeseries"
    _UNKNOWN = "__PRIORITYRESOURCE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PriorityResource":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        workouts: typing.Callable[[], T_Result],
        activity: typing.Callable[[], T_Result],
        sleep: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        timeseries: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PriorityResource.WORKOUTS:
            return workouts()
        if self is PriorityResource.ACTIVITY:
            return activity()
        if self is PriorityResource.SLEEP:
            return sleep()
        if self is PriorityResource.BODY:
            return body()
        if self is PriorityResource.TIMESERIES:
            return timeseries()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EventDestinationPreferencesEnabledItem(str, enum.Enum):
    CLOUD_PUBSUB = "cloud_pubsub"
    RABBITMQ = "rabbitmq"
    SVIX = "svix"
    AZURE_AMQP = "azure_amqp"
    _UNKNOWN = "__EVENTDESTINATIONPREFERENCESENABLEDITEM_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "EventDestinationPreferencesEnabledItem":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        cloud_pubsub: typing.Callable[[], T_Result],
        rabbitmq: typing.Callable[[], T_Result],
        svix: typing.Callable[[], T_Result],
        azure_amqp: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is EventDestinationPreferencesEnabledItem.CLOUD_PUBSUB:
            return cloud_pubsub()
        if self is EventDestinationPreferencesEnabledItem.RABBITMQ:
            return rabbitmq()
        if self is EventDestinationPreferencesEnabledItem.SVIX:
            return svix()
        if self is EventDestinationPreferencesEnabledItem.AZURE_AMQP:
            return azure_amqp()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class SexualOrientation(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    LESBIAN_GAY_OR_HOMOSEXUAL = "lesbian_gay_or_homosexual"
    HETEROSEXUAL_OR_STRAIGHT = "heterosexual_or_straight"
    BISEXUAL = "bisexual"
    DONT_KNOW = "dont_know"
    OTHER = "other"
    _UNKNOWN = "__SEXUALORIENTATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "SexualOrientation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        lesbian_gay_or_homosexual: typing.Callable[[], T_Result],
        heterosexual_or_straight: typing.Callable[[], T_Result],
        bisexual: typing.Callable[[], T_Result],
        dont_know: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is SexualOrientation.LESBIAN_GAY_OR_HOMOSEXUAL:
            return lesbian_gay_or_homosexual()
        if self is SexualOrientation.HETEROSEXUAL_OR_STRAIGHT:
            return heterosexual_or_straight()
        if self is SexualOrientation.BISEXUAL:
            return bisexual()
        if self is SexualOrientation.DONT_KNOW:
            return dont_know()
        if self is SexualOrientation.OTHER:
            return other()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_facing_source import ClientFacingSource


class ClientFacingBody(UniversalBaseModel):
    id: str
    user_id: str = pydantic.Field()
    """
    User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.
    """

    date: dt.datetime = pydantic.Field()
    """
    Date of the specified record, formatted as ISO8601 datetime string in UTC 00:00. Deprecated in favour of calendar_date.
    """

    calendar_date: str = pydantic.Field()
    """
    Date of the summary in the YYYY-mm-dd format.
    """

    weight: typing.Optional[float] = pydantic.Field(default=None)
    """
    Weight in kg::kg
    """

    fat: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total body fat percentage::perc
    """

    water_percentage: typing.Optional[float] = pydantic.Field(default=None)
    """
    Water percentage in the body::perc
    """

    muscle_mass_percentage: typing.Optional[float] = pydantic.Field(default=None)
    """
    Muscle mass percentage in the body::perc
    """

    visceral_fat_index: typing.Optional[float] = pydantic.Field(default=None)
    """
    Visceral fat index::scalar
    """

    bone_mass_percentage: typing.Optional[float] = pydantic.Field(default=None)
    """
    Bone mass percentage in the body::perc
    """

    body_mass_index: typing.Optional[float] = None
    lean_body_mass_kilogram: typing.Optional[float] = None
    waist_circumference_centimeter: typing.Optional[float] = None
    source: ClientFacingSource
    created_at: dt.datetime
    updated_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .clinical_information import ClinicalInformation
from .performing_laboratory import PerformingLaboratory
from .sample_data_date_collected import SampleDataDateCollected
from .sample_data_date_received import SampleDataDateReceived
from .sample_data_date_reported import SampleDataDateReported


class SampleData(UniversalBaseModel):
    sample_id: typing.Optional[str] = None
    control_number: typing.Optional[str] = None
    date_collected: typing.Optional[SampleDataDateCollected] = None
    date_received: typing.Optional[SampleDataDateReceived] = None
    date_reported: typing.Optional[SampleDataDateReported] = None
    performing_laboratories: typing.Optional[typing.Dict[str, typing.Optional[PerformingLaboratory]]] = None
    clinical_information: typing.Optional[ClinicalInformation] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.client_facing_meal_response import ClientFacingMealResponse
from ..types.http_validation_error import HttpValidationError


class RawMealClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ClientFacingMealResponse]:
        """
        Get user's meals

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ClientFacingMealResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/summary/meal/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "provider": provider,
                "start_date": start_date,
                "end_date": end_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClientFacingMealResponse,
                    parse_obj_as(
                        type_=ClientFacingMealResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMealClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ClientFacingMealResponse]:
        """
        Get user's meals

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ClientFacingMealResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/summary/meal/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "provider": provider,
                "start_date": start_date,
                "end_date": end_date,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ClientFacingMealResponse,
                    parse_obj_as(
                        type_=ClientFacingMealResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.client_facing_sleep_stream import ClientFacingSleepStream
from ..types.client_sleep_response import ClientSleepResponse
from ..types.raw_sleep import RawSleep
from .raw_client import AsyncRawSleepClient, RawSleepClient


class SleepClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSleepClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSleepClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSleepClient
        """
        return self._raw_client

    def get(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientSleepResponse:
        """
        Get sleep summary for user_id

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientSleepResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.sleep.get(user_id='user_id', start_date='start_date', )
        """
        _response = self._raw_client.get(
            user_id, start_date=start_date, provider=provider, end_date=end_date, request_options=request_options
        )
        return _response.data

    def get_raw(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RawSleep:
        """
        Get raw sleep summary for user_id

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RawSleep
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.sleep.get_raw(user_id='user_id', start_date='start_date', )
        """
        _response = self._raw_client.get_raw(
            user_id, start_date=start_date, provider=provider, end_date=end_date, request_options=request_options
        )
        return _response.data

    def get_stream_by_sleep_id(
        self, sleep_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingSleepStream:
        """
        Get Sleep stream for a user_id

        Parameters
        ----------
        sleep_id : str
            The Vital Sleep ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingSleepStream
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.sleep.get_stream_by_sleep_id(sleep_id='sleep_id', )
        """
        _response = self._raw_client.get_stream_by_sleep_id(sleep_id, request_options=request_options)
        return _response.data


class AsyncSleepClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSleepClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSleepClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSleepClient
        """
        return self._raw_client

    async def get(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientSleepResponse:
        """
        Get sleep summary for user_id

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientSleepResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.sleep.get(user_id='user_id', start_date='start_date', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            user_id, start_date=start_date, provider=provider, end_date=end_date, request_options=request_options
        )
        return _response.data

    async def get_raw(
        self,
        user_id: str,
        *,
        start_date: str,
        provider: typing.Optional[str] = None,
        end_date: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RawSleep:
        """
        Get raw sleep summary for user_id

        Parameters
        ----------
        user_id : str

        start_date : str
            Date from in YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 00:00:00

        provider : typing.Optional[str]
            Provider oura/strava etc

        end_date : typing.Optional[str]
            Date to YYYY-MM-DD or ISO formatted date time. If a date is provided without a time, the time will be set to 23:59:59

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RawSleep
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.sleep.get_raw(user_id='user_id', start_date='start_date', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_raw(
            user_id, start_date=start_date, provider=provider, end_date=end_date, request_options=request_options
        )
        return _response.data

    async def get_stream_by_sleep_id(
        self, sleep_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingSleepStream:
        """
        Get Sleep stream for a user_id

        Parameters
        ----------
        sleep_id : str
            The Vital Sleep ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingSleepStream
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.sleep.get_stream_by_sleep_id(sleep_id='sleep_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_stream_by_sleep_id(sleep_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .address import Address
from .ethnicity import Ethnicity
from .gender_identity import GenderIdentity
from .guarantor_details import GuarantorDetails
from .race import Race
from .sexual_orientation import SexualOrientation


class UserInfo(UniversalBaseModel):
    first_name: str
    last_name: str
    email: str
    phone_number: str
    gender: str
    dob: str
    address: Address
    medical_proxy: typing.Optional[GuarantorDetails] = None
    race: typing.Optional[Race] = None
    ethnicity: typing.Optional[Ethnicity] = None
    sexual_orientation: typing.Optional[SexualOrientation] = None
    gender_identity: typing.Optional[GenderIdentity] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DiscreteTimeseriesExprTimeseries(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    GLUCOSE = "glucose"
    HEARTRATE = "heartrate"
    HRV = "hrv"
    IGE = "ige"
    IGG = "igg"
    CHOLESTEROL = "cholesterol"
    WEIGHT = "weight"
    FAT = "fat"
    BLOOD_OXYGEN = "blood_oxygen"
    ELECTROCARDIOGRAM_VOLTAGE = "electrocardiogram_voltage"
    RESPIRATORY_RATE = "respiratory_rate"
    STRESS_LEVEL = "stress_level"
    _UNKNOWN = "__DISCRETETIMESERIESEXPRTIMESERIES_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DiscreteTimeseriesExprTimeseries":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        glucose: typing.Callable[[], T_Result],
        heartrate: typing.Callable[[], T_Result],
        hrv: typing.Callable[[], T_Result],
        ige: typing.Callable[[], T_Result],
        igg: typing.Callable[[], T_Result],
        cholesterol: typing.Callable[[], T_Result],
        weight: typing.Callable[[], T_Result],
        fat: typing.Callable[[], T_Result],
        blood_oxygen: typing.Callable[[], T_Result],
        electrocardiogram_voltage: typing.Callable[[], T_Result],
        respiratory_rate: typing.Callable[[], T_Result],
        stress_level: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DiscreteTimeseriesExprTimeseries.GLUCOSE:
            return glucose()
        if self is DiscreteTimeseriesExprTimeseries.HEARTRATE:
            return heartrate()
        if self is DiscreteTimeseriesExprTimeseries.HRV:
            return hrv()
        if self is DiscreteTimeseriesExprTimeseries.IGE:
            return ige()
        if self is DiscreteTimeseriesExprTimeseries.IGG:
            return igg()
        if self is DiscreteTimeseriesExprTimeseries.CHOLESTEROL:
            return cholesterol()
        if self is DiscreteTimeseriesExprTimeseries.WEIGHT:
            return weight()
        if self is DiscreteTimeseriesExprTimeseries.FAT:
            return fat()
        if self is DiscreteTimeseriesExprTimeseries.BLOOD_OXYGEN:
            return blood_oxygen()
        if self is DiscreteTimeseriesExprTimeseries.ELECTROCARDIOGRAM_VOLTAGE:
            return electrocardiogram_voltage()
        if self is DiscreteTimeseriesExprTimeseries.RESPIRATORY_RATE:
            return respiratory_rate()
        if self is DiscreteTimeseriesExprTimeseries.STRESS_LEVEL:
            return stress_level()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .provider_mfa_request_method import ProviderMfaRequestMethod


class ProviderMfaRequest(UniversalBaseModel):
    method: ProviderMfaRequestMethod = pydantic.Field()
    """
    The MFA method requested by the password provider to complete authentication. ℹ️ This enum is non-exhaustive.
    """

    hint: str = pydantic.Field()
    """
    The MFA hint provided by the password provider, e.g., the redacted phone number.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .historical_pull_completed import HistoricalPullCompleted


class ClientFacingMindfulnessMinutesHistoricalPullCompleted(UniversalBaseModel):
    event_type: typing.Literal["historical.data.mindfulness_minutes.created"] = (
        "historical.data.mindfulness_minutes.created"
    )
    user_id: str
    client_user_id: str
    team_id: str
    data: HistoricalPullCompleted

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
